# coding: utf-8
# Copyright (c) 2016, 2025, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20190506


from oci.util import formatted_flat_dict, NONE_SENTINEL, value_allowed_none_or_none_sentinel  # noqa: F401
from oci.decorators import init_model_state_from_kwargs


@init_model_state_from_kwargs
class Entity(object):
    """
    Metadata for an entity.
    """

    #: A constant which can be used with the type property of a Entity.
    #: This constant has a value of "COMPOSITE"
    TYPE_COMPOSITE = "COMPOSITE"

    #: A constant which can be used with the type property of a Entity.
    #: This constant has a value of "ENUM_VALUES"
    TYPE_ENUM_VALUES = "ENUM_VALUES"

    def __init__(self, **kwargs):
        """
        Initializes a new Entity object with values from keyword arguments. This class has the following subclasses and if you are using this class as input
        to a service operations then you should favor using a subclass over the base class:

        * :class:`~oci.oda.models.CompositeEntity`
        * :class:`~oci.oda.models.ValueListEntity`

        The following keyword arguments are supported (corresponding to the getters/setters of this class):

        :param id:
            The value to assign to the id property of this Entity.
        :type id: str

        :param name:
            The value to assign to the name property of this Entity.
        :type name: str

        :param type:
            The value to assign to the type property of this Entity.
            Allowed values for this property are: "COMPOSITE", "ENUM_VALUES"
        :type type: str

        """
        self.swagger_types = {
            'id': 'str',
            'name': 'str',
            'type': 'str'
        }

        self.attribute_map = {
            'id': 'id',
            'name': 'name',
            'type': 'type'
        }

        self._id = None
        self._name = None
        self._type = None

    @staticmethod
    def get_subtype(object_dictionary):
        """
        Given the hash representation of a subtype of this class,
        use the info in the hash to return the class of the subtype.
        """
        type = object_dictionary['type']

        if type == 'COMPOSITE':
            return 'CompositeEntity'

        if type == 'ENUM_VALUES':
            return 'ValueListEntity'
        else:
            return 'Entity'

    @property
    def id(self):
        """
        **[Required]** Gets the id of this Entity.
        Unique immutable identifier that was assigned when the resource was created.


        :return: The id of this Entity.
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """
        Sets the id of this Entity.
        Unique immutable identifier that was assigned when the resource was created.


        :param id: The id of this Entity.
        :type: str
        """
        self._id = id

    @property
    def name(self):
        """
        **[Required]** Gets the name of this Entity.
        The entity name. This must be unique within the parent resource.


        :return: The name of this Entity.
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """
        Sets the name of this Entity.
        The entity name. This must be unique within the parent resource.


        :param name: The name of this Entity.
        :type: str
        """
        self._name = name

    @property
    def type(self):
        """
        **[Required]** Gets the type of this Entity.
        Type of skill entity.

        Allowed values for this property are: "COMPOSITE", "ENUM_VALUES"


        :return: The type of this Entity.
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type):
        """
        Sets the type of this Entity.
        Type of skill entity.


        :param type: The type of this Entity.
        :type: str
        """
        allowed_values = ["COMPOSITE", "ENUM_VALUES"]
        if not value_allowed_none_or_none_sentinel(type, allowed_values):
            raise ValueError(
                f"Invalid value for `type`, must be None or one of {allowed_values}"
            )
        self._type = type

    def __repr__(self):
        return formatted_flat_dict(self)

    def __eq__(self, other):
        if other is None:
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other
